@@ -18,6 +18,6 @@ int fly_logtime(char *buffer, int bufsize, fly_time_t *t);
 int fly_imt_fixdate(char *buf, size_t buflen, time_t *time);
 
 #define FLY_IMT_FIXDATE_FORMAT			("%a, %d %b %Y %H:%M:%S GMT")
-int fly_cmp_imt_fixdate(char *t1, __unused size_t t1_len, char *t2, __unused size_t t2_len);
+int fly_cmp_imt_fixdate(char *t1, __fly_unused size_t t1_len, char *t2, __fly_unused size_t t2_len);
 
 #endif
