@@ -15,7 +15,7 @@ struct __fly_lang{
 	fly_bit_t				asterisk: 1;
 };
 
-__unused static inline const char *fly_lang_name(struct __fly_lang *__l)
+__fly_unused static inline const char *fly_lang_name(struct __fly_lang *__l)
 {
 	return __l->asterisk ? FLY_LANG_ASTERISK : __l->lname;
 }
@@ -29,7 +29,11 @@ struct fly_lang{
 };
 typedef struct fly_lang fly_lang_t;
 
+#define FLY_ACCEPT_LANG_SUCCESS				0
+#define FLY_ACCEPT_LANG_SYNTAX_ERROR		-1
+#define FLY_ACCEPT_LANG_ERROR				-2
+#define FLY_ACCEPT_LANG_NOT_ACCEPTABLE		-3
 int fly_accept_language(fly_request_t *req);
-#define FLY_ACCEPT_LANG					("Accept-Language")
+#define FLY_ACCEPT_LANG					("accept-language")
 
 #endif
