@@ -33,6 +33,10 @@ bool fly_mime_invalid(fly_mime_type_t *type)
 
 static void __fly_mime_init(fly_request_t *req);
 __fly_static int __fly_add_accept_mime(fly_mime_t *m, struct __fly_mime *nm);
+#define __FLY_ACCEPT_PARSE_MIME_SUCCESS				1
+#define __FLY_ACCEPT_PARSE_MIME_ERROR				-1
+#define __FLY_ACCEPT_PARSE_MIME_SYNTAX_ERROR		-2
+#define __FLY_ACCEPT_PARSE_MIME_NOT_ACCEPTABLE		-3
 __fly_static int __fly_accept_parse(fly_mime_t *mime, fly_hdr_c *c);
 __fly_static int __fly_accept_mime_parse(fly_mime_t *mime, fly_hdr_value *value);
 #define FLY_MIME_PARSE_ERROR	-1
@@ -173,25 +177,36 @@ __fly_static int __fly_add_accept_asterisk(fly_request_t *req)
 	return __fly_add_accept_mime(req->mime, am);
 }
 
-int fly_accept_mime(__unused fly_request_t *request)
+int fly_accept_mime(__fly_unused fly_request_t *request)
 {
 	fly_hdr_ci *header;
 	fly_hdr_c  *accept;
 
 	header = request->header;
 	if (fly_unlikely_null(request) || fly_unlikely_null(request->pool) || fly_unlikely_null(request->header))
-		return -1;
+		return FLY_ACCEPT_MIME_ERROR;
 
 	__fly_mime_init(request);
 	switch(__fly_accept_mime(header, &accept)){
 	case __FLY_ACCEPT_MIME_ERROR:
-		return -1;
+		return FLY_ACCEPT_MIME_ERROR;
 	case __FLY_ACCEPT_MIME_NOTFOUND:
 		if (__fly_add_accept_asterisk(request) == -1)
-			return -1;
-		return 0;
+			return FLY_ACCEPT_MIME_ERROR;
+		return FLY_ACCEPT_MIME_SUCCESS;
 	case __FLY_ACCEPT_MIME_FOUND:
-		return __fly_accept_parse(request->mime, accept);
+		switch(__fly_accept_parse(request->mime, accept)){
+		case __FLY_ACCEPT_PARSE_MIME_SUCCESS:
+			return FLY_ACCEPT_MIME_SUCCESS;
+		case __FLY_ACCEPT_PARSE_MIME_ERROR:
+			return FLY_ACCEPT_MIME_ERROR;
+		case __FLY_ACCEPT_PARSE_MIME_SYNTAX_ERROR:
+			return FLY_ACCEPT_MIME_SYNTAX_ERROR;
+		case __FLY_ACCEPT_PARSE_MIME_NOT_ACCEPTABLE:
+			return FLY_ACCEPT_MIME_NOT_ACCEPTABLE;
+		default:
+			FLY_NOT_COME_HERE
+		}
 	default:
 		FLY_NOT_COME_HERE
 	}
@@ -342,7 +357,7 @@ __fly_static int __fly_accept_mime_parse(fly_mime_t *mime, fly_hdr_value *value)
 	int decimal_places = 0;
 	struct __fly_mime *__nm;
 
-	__unused enum{
+	__fly_unused enum{
 		MEDIA_RANGE,
 		ACCEPT_PARAMS,
 	} pfase;
@@ -894,7 +909,7 @@ static inline char __fly_lu_ignore(char c)
 	return __fly_ualpha(c) ? (c+0x20) : c;
 }
 
-__unused __fly_static int __fly_same_type(char *t1, char *t2)
+__fly_unused __fly_static int __fly_same_type(char *t1, char *t2)
 {
 	while(!__fly_slash(*t2))
 		if (__fly_lu_ignore(*t1++) != __fly_lu_ignore(*t2++))
@@ -903,7 +918,7 @@ __unused __fly_static int __fly_same_type(char *t1, char *t2)
 	return 0;
 }
 
-__unused __fly_static int __fly_same_type_n(const char *t1, const char *t2, size_t n)
+__fly_unused __fly_static int __fly_same_type_n(const char *t1, const char *t2, size_t n)
 {
 	size_t i=0;
 	while(__fly_lu_ignore(*t1++) == __fly_lu_ignore(*t2++))
@@ -912,7 +927,7 @@ __unused __fly_static int __fly_same_type_n(const char *t1, const char *t2, size
 
 	return 0;
 }
-__unused __fly_static int __fly_copy_type(char *dist, char *src)
+__fly_unused __fly_static int __fly_copy_type(char *dist, char *src)
 {
 	int i=0;
 	while(!__fly_slash(*src)){
@@ -925,7 +940,7 @@ __unused __fly_static int __fly_copy_type(char *dist, char *src)
 	return 0;
 }
 
-__unused __fly_static int __fly_copy_subtype(char *dist, char *src)
+__fly_unused __fly_static int __fly_copy_subtype(char *dist, char *src)
 {
 	int i=0;
 	while(__fly_subtype(*src)){
@@ -996,26 +1011,36 @@ __fly_static int __fly_accept_qvalue_from_str(fly_hdr_value *qvalue)
 
 __fly_static int __fly_accept_parse(fly_mime_t *mime, fly_hdr_c *c)
 {
+	struct fly_bllist *__b;
+	struct __fly_mime *__m;
 	fly_hdr_value *value;
 
-	if (!mime)
-		return -1;
-
 	if (!c->value)
-		return FLY_MIME_PARSE_ERROR;
-	value = c->value;
+		return __FLY_ACCEPT_PARSE_MIME_SYNTAX_ERROR;
 
+	value = c->value;
 	switch (__fly_accept_mime_parse(mime, value)){
 	case FLY_MIME_PARSE_SUCCESS:
-		return 1;
+		break;
 	case FLY_MIME_PARSE_ERROR:
-		return -1;
+		return __FLY_ACCEPT_PARSE_MIME_ERROR;
 	case FLY_MIME_PARSE_PERROR:
-		return 0;
+		return __FLY_ACCEPT_PARSE_MIME_SYNTAX_ERROR;
 	default:
-		return -1;
+		FLY_NOT_COME_HERE
 	}
-	FLY_NOT_COME_HERE
+
+	if (mime->accept_count == 0)
+		return __FLY_ACCEPT_PARSE_MIME_SYNTAX_ERROR;
+
+	fly_for_each_bllist(__b, &mime->accepts){
+		__m = fly_bllist_data(__b, struct __fly_mime, blelem);
+		if (__m->quality_value > 0)
+			return __FLY_ACCEPT_PARSE_MIME_SUCCESS;
+	}
+
+	/* There is no mime type that have quality value greater than 0 */
+	return __FLY_ACCEPT_PARSE_MIME_NOT_ACCEPTABLE;
 }
 
 __fly_static void __fly_mime_param_add(struct __fly_mime *__nm, struct __fly_accept_param *param)
@@ -1032,7 +1057,7 @@ __fly_static void __fly_mime_ext_add(struct __fly_mime *__ne, struct __fly_accep
 	return;
 }
 
-__unused __fly_static void __fly_param_copyl(char *dist, char *src, size_t maxlen)
+__fly_unused __fly_static void __fly_param_copyl(char *dist, char *src, size_t maxlen)
 {
 	size_t i=0;
 	while(__fly_token(*src) && !__fly_equal(*src)){
@@ -1044,7 +1069,7 @@ __unused __fly_static void __fly_param_copyl(char *dist, char *src, size_t maxle
 	*dist = '\0';
 }
 
-__unused __fly_static void __fly_param_copyr(char *dist, char *src, size_t maxlen)
+__fly_unused __fly_static void __fly_param_copyr(char *dist, char *src, size_t maxlen)
 {
 	size_t i=0;
 	while(__fly_token(*src)){
@@ -1056,7 +1081,7 @@ __unused __fly_static void __fly_param_copyr(char *dist, char *src, size_t maxle
 	*dist = '\0';
 }
 
-__unused __fly_static void __fly_param_copyql(char *dist, char *src, size_t maxlen)
+__fly_unused __fly_static void __fly_param_copyql(char *dist, char *src, size_t maxlen)
 {
 	size_t i=0;
 	while(__fly_token(*src) && !__fly_equal(*src)){
@@ -1068,7 +1093,7 @@ __unused __fly_static void __fly_param_copyql(char *dist, char *src, size_t maxl
 	*dist = '\0';
 }
 
-__unused __fly_static void __fly_param_copyqr(char *dist, char *src, size_t maxlen)
+__fly_unused __fly_static void __fly_param_copyqr(char *dist, char *src, size_t maxlen)
 {
 	size_t i=0;
 	while(__fly_token(*src) && !__fly_quoted(*src)){
@@ -1080,7 +1105,7 @@ __unused __fly_static void __fly_param_copyqr(char *dist, char *src, size_t maxl
 	}
 	*dist = '\0';
 }
-__unused __fly_static void  __fly_ext_param_copyl(char *dist, char *src, size_t maxlen)
+__fly_unused __fly_static void  __fly_ext_param_copyl(char *dist, char *src, size_t maxlen)
 {
 	size_t i=0;
 	while(__fly_token(*src) && !__fly_equal(*src)){
@@ -1091,7 +1116,7 @@ __unused __fly_static void  __fly_ext_param_copyl(char *dist, char *src, size_t
 	}
 	*dist = '\0';
 }
-__unused __fly_static void  __fly_ext_param_copyr(char *dist, char *src, size_t maxlen)
+__fly_unused __fly_static void  __fly_ext_param_copyr(char *dist, char *src, size_t maxlen)
 {
 	size_t i=0;
 	while(__fly_token(*src)){
