@@ -1,5 +1,9 @@
 #include "pyserver.h"
 #include "v2.h"
+#ifdef DEBUG
+#include "log.h"
+#endif
+#include "err.h"
 
 struct __pyfly_server{
 	PyObject_HEAD
@@ -9,9 +13,19 @@ struct __pyfly_server{
 	long 				worker;
 	long 				reqworker;
 	bool 				ssl;
-	const char			*log;
-	const char			*ssl_crt_path;
-	const char 			*ssl_key_path;
+	char				*log;
+	char				*ssl_crt_path;
+	char 				*ssl_key_path;
+	bool				log_stdout;
+	bool				log_stderr;
+	long				backlog;
+	long				max_response_content_length;
+	long				max_request_content_length;
+	long				request_timeout;
+	const char			*index_path;
+	const char			*default_content_path;
+	struct fly_err		err;
+	long				encoding_threshold;
 };
 
 struct PyMemberDef __pyfly_server_members[] = {
@@ -23,6 +37,15 @@ struct PyMemberDef __pyfly_server_members[] = {
 	{"_ssl", T_BOOL, offsetof(struct __pyfly_server, ssl), READONLY, ""},
 	{"_ssl_crt_path", T_STRING, offsetof(struct __pyfly_server, ssl_crt_path), READONLY, ""},
 	{"_ssl_key_path", T_STRING, offsetof(struct __pyfly_server, ssl_key_path), READONLY, ""},
+	{"_log_stdout", T_BOOL, offsetof(struct __pyfly_server, log_stdout), READONLY, ""},
+	{"_log_stderr", T_BOOL, offsetof(struct __pyfly_server, log_stderr), READONLY, ""},
+	{"_backlog", T_LONG, offsetof(struct __pyfly_server, backlog), READONLY, ""},
+	{"_max_response_content_length", T_LONG, offsetof(struct __pyfly_server, max_response_content_length), READONLY, ""},
+	{"_max_request_content_length", T_LONG, offsetof(struct __pyfly_server, max_request_content_length), READONLY, ""},
+	{"_request_timeout", T_LONG, offsetof(struct __pyfly_server, request_timeout), READONLY, ""},
+	{"_index_path", T_STRING, offsetof(struct __pyfly_server, index_path), READONLY, ""},
+	{"_default_content_path", T_STRING, offsetof(struct __pyfly_server, default_content_path), READONLY, ""},
+	{"_encoding_threshold", T_LONG, offsetof(struct __pyfly_server, encoding_threshold), READONLY, ""},
 	{NULL}
 };
 
@@ -148,51 +171,53 @@ typedef struct __pyfly_server __pyfly_server_t;
 
 static void __pyfly_server_dealloc(__pyfly_server_t *self)
 {
-	if (self->master)
-		fly_master_release(self->master);
-
-//	fly_remove_pidfile();
 	Py_TYPE(self)->tp_free((PyObject *) self);
 }
 
 
-__unused static PyObject *__pyfly_server_new(PyTypeObject *type, PyObject *args, PyObject *kwargs)
+__fly_unused static PyObject *__pyfly_server_new(PyTypeObject *type, PyObject *args, PyObject *kwargs)
 {
 	__pyfly_server_t *self;
 	self = (__pyfly_server_t *) type->tp_alloc(type, 0);
 	return (PyObject *) self;
 }
 
-static int pyfly_parse_config_file(void)
+static int pyfly_parse_config_file(struct fly_err *err)
 {
 	int res;
 
-	res = fly_parse_config_file();
+	memset(err, '\0', sizeof(struct fly_err));
+	res = fly_parse_config_file(err);
+#ifdef DEBUG
+	printf("PyFLY_PARSE_CONFIG_FILE: %s\n", err->content);
+#endif
 	switch (res){
-	case FLY_PARSE_CONFIG_NOTFOUND:
-		break;
-	case FLY_PARSE_CONFIG_ERROR:
-		PyErr_SetString(PyExc_ValueError, "parse file error.");
-		break;
 	case FLY_PARSE_CONFIG_SUCCESS:
 		break;
+	case FLY_PARSE_CONFIG_NOTFOUND:
+		PyErr_WarnFormat(PyExc_ResourceWarning, 1, "%s", (const char *) err->content);
+		break;
+	case FLY_PARSE_CONFIG_ERROR:
+		PyErr_Format(PyExc_ValueError, "Parse config error. %s", err->content);
+		break;
 	case FLY_PARSE_CONFIG_SYNTAX_ERROR:
-		PyErr_SetString(PyExc_ValueError, "parse file error.");
+		PyErr_Format(PyExc_ValueError, "Parse syntax config error.");
 		break;
 	default:
-		PyErr_SetString(PyExc_ValueError, "parse file error.");
-		break;
+		FLY_NOT_COME_HERE
 	}
 	return res;
 }
 
 static fly_master_t *pyfly_master_init(void)
 {
+	struct fly_err err;
 	fly_master_t *res;
 
-	res = fly_master_init();
+	res = fly_master_init(&err);
 	if (res == NULL)
-		PyErr_Format(PyExc_Exception, "master init error. (\"%s\")", strerror(errno));
+		PyErr_Format(PyExc_Exception,
+			"Master init. %s", err.content);
 
 	return res;
 }
@@ -246,7 +271,7 @@ extern PyTypeObject FlyResponseType;
 #define PYFLY_HEADER_KEY						"header"
 #define PYFLY_STATUS_CODE_KEY					"status_code"
 #define PYFLY_CONTENT_TYPE_KEY					"content_type"
-static fly_response_t *pyfly_route_handler(fly_request_t *request, void *data)
+static fly_response_t *pyfly_route_handler(fly_request_t *request, fly_route_t *route, void *data)
 {
 #define PYFLY_RESHANDLER_ARGS_COUNT			1
 	PyObject *__func, *__args, *__reqdict, *pyres=NULL, *__margs;
@@ -307,7 +332,7 @@ static fly_response_t *pyfly_route_handler(fly_request_t *request, void *data)
 
 	Py_DECREF(__margs);
 	Py_DECREF(__pyscheme);
-	/* peer connection info */
+	/* Peer connection info */
 	PyObject *__pyhost = PyUnicode_FromString((const char *) request->connect->hostname);
 	if (PyDict_SetItemString(__reqdict, "host", (PyObject *) __pyhost) == -1)
 		return NULL;
@@ -318,7 +343,43 @@ static fly_response_t *pyfly_route_handler(fly_request_t *request, void *data)
 
 	Py_DECREF(__pyhost);
 	Py_DECREF(__pyport);
-	/* header */
+
+	/* Path parameter */
+	if (fly_path_param_count(&reqline->path_params) > 0){
+		struct fly_param *__pp;
+		int i=0;
+		PyListObject *__pyparams = (PyListObject *) PyList_New((Py_ssize_t) fly_path_param_count(&reqline->path_params));
+		fly_for_each_bllist(__b, &reqline->path_params.params){
+			__pp = fly_bllist_data(__b, struct fly_param, blelem);
+
+			PyDictObject *__pd = (PyDictObject *) PyDict_New();
+			PyObject *__n, *type, *value;
+
+			__n = PyUnicode_FromStringAndSize(__pp->var_ptr, __pp->var_len);
+			if (__n == NULL || PyDict_SetItemString((PyObject *) __pd, "param", __n) == -1)
+				return NULL;
+
+			type = PyLong_FromLong((long) __pp->type);
+			if (type == NULL || PyDict_SetItemString((PyObject *) __pd, "type", type) == -1)
+				return NULL;
+
+			value = PyUnicode_FromStringAndSize((const char *) __pp->value, (Py_ssize_t) __pp->value_len);
+			if (value == NULL || PyDict_SetItemString((PyObject *) __pd, "value", value) == -1)
+				return NULL;
+
+
+			if (PyList_SetItem((PyObject *) __pyparams, i, (PyObject *) __pd) == -1)
+				return NULL;
+			Py_DECREF(__n);
+			Py_DECREF(type);
+			Py_DECREF(value);
+			i++;
+		}
+		if (PyDict_SetItemString((PyObject *) __reqdict, "path_params", (PyObject *) __pyparams) == -1)
+			return NULL;
+		Py_DECREF(__pyparams);
+	}
+	/* Header */
 	if (request->header){
 		header = request->header;
 		PyListObject *__pylist = (PyListObject *) PyList_New((Py_ssize_t) header->chain_count);
@@ -341,44 +402,48 @@ static fly_response_t *pyfly_route_handler(fly_request_t *request, void *data)
 				return NULL;
 
 			if (__c->cookie){
-				PyDictObject *__cookie = (PyDictObject *) PyDict_New();
-				PyObject *__cn=NULL, *__cv=NULL;
-				char *__cn_ptr=NULL, *__cv_ptr=NULL, *np;
-				size_t __cn_len=0, __cv_len=0;
+				PyObject *__cookie;
 
-				np = __c->value;
-
-				__cn_ptr = __c->value;
-				while(!fly_equal(*np) && \
-						np<__c->value+__c->value_len-1){
-					np++;
-				}
-
-				__cn_len = np-__cn_ptr;
-				if (++np<=__c->value+__c->value_len-1){
-					__cv_ptr = np;
-					__cv_len = (size_t) ((__c->value+__c->value_len)-np);
-#ifdef DEBUG
-					assert(__cv_len>0);
-#endif
-				}
-
-				__cn = PyUnicode_FromStringAndSize(__cn_ptr, __cn_len);
-				if (PyDict_SetItemString((PyObject *) __cookie, "name", __cn) == -1)
-					return NULL;
-
-				if (__cv_ptr){
-					__cv = PyUnicode_FromStringAndSize(__cv_ptr, __cv_len);
-					if (PyDict_SetItemString((PyObject *) __cookie, "value", __cv) == -1)
-						return NULL;
-				}
+				__cookie = PyUnicode_FromStringAndSize(__c->value, __c->value_len);
+//				PyDictObject *__cookie = (PyDictObject *) PyDict_New();
+//				PyObject *__cn=NULL, *__cv=NULL;
+//				char *__cn_ptr=NULL, *__cv_ptr=NULL, *np;
+//				size_t __cn_len=0, __cv_len=0;
+//
+//				/* FIXME: INVALID PARSE */
+//				np = __c->value;
+//				__cn_ptr = __c->value;
+//				while(!fly_equal(*np) && \
+//						np<__c->value+__c->value_len-1){
+//					np++;
+//				}
+//
+//				__cn_len = np-__cn_ptr;
+//				if (++np<=__c->value+__c->value_len-1){
+//					__cv_ptr = np;
+//					__cv_len = (size_t) ((__c->value+__c->value_len)-np);
+//#ifdef DEBUG
+//					assert(__cv_len>0);
+//#endif
+//				}
+//
+//				__cn = PyUnicode_FromStringAndSize(__cn_ptr, __cn_len);
+//				if (PyDict_SetItemString((PyObject *) __cookie, "name", __cn) == -1)
+//					return NULL;
+//
+//				if (__cv_ptr){
+//					__cv = PyUnicode_FromStringAndSize(__cv_ptr, __cv_len);
+//					if (PyDict_SetItemString((PyObject *) __cookie, "value", __cv) == -1)
+//						return NULL;
+//				}
 
 				if (PyList_Append((PyObject *) __pycookie, (PyObject *) __cookie) == -1)
 					return NULL;
 
-				Py_DECREF(__cn);
-				if (__cv != NULL)
-					Py_DECREF(__cv);
+				Py_DECREF(__cookie);
+//				Py_DECREF(__cn);
+//				if (__cv != NULL)
+//					Py_DECREF(__cv);
 			}
 			i++;
 			Py_DECREF(__n);
@@ -903,23 +968,23 @@ static PyObject *__pyfly_configure(__pyfly_server_t *self, PyObject *args)
 		Py_INCREF(routes);
 
 	if (!PyList_Check(routes)){
-		PyErr_SetString(PyExc_ValueError, "routes must be list type.");
+		PyErr_SetString(PyExc_ValueError, "Routes must be list type.");
 		goto error;
 	}
 
 	if (config_path && setenv(FLY_CONFIG_PATH, config_path, 1) == -1){
-		PyErr_SetString(PyExc_ValueError, "environment setting error.");
+		PyErr_SetString(PyExc_ValueError, "Environment setting error.");
 		goto error;
 	}
 
 	/* config setting */
-	switch (pyfly_parse_config_file()){
+	switch (pyfly_parse_config_file(&self->err)){
+	case FLY_PARSE_CONFIG_SUCCESS:
+		break;
 	case FLY_PARSE_CONFIG_NOTFOUND:
 		break;
 	case FLY_PARSE_CONFIG_ERROR:
 		goto error;
-	case FLY_PARSE_CONFIG_SUCCESS:
-		break;
 	case FLY_PARSE_CONFIG_SYNTAX_ERROR:
 		goto error;
 	default:
@@ -943,6 +1008,7 @@ static PyObject *__pyfly_configure(__pyfly_server_t *self, PyObject *args)
 		for (Py_ssize_t j=0; j<routes_len; j++){
 			PyObject *__d, *uri, *func, *method;
 			const char *uri_c, *method_c;
+			Py_ssize_t uri_size;
 			fly_method_e *m;
 
 			__d = PyList_GetItem(routes, j);
@@ -968,7 +1034,9 @@ static PyObject *__pyfly_configure(__pyfly_server_t *self, PyObject *args)
 				goto error;
 			}
 
-			uri_c = PyUnicode_AsUTF8AndSize(uri, NULL);
+			/* NULL termination */
+			uri_c = PyUnicode_AsUTF8AndSize(uri, &uri_size);
+			/* NULL termination */
 			method_c = PyUnicode_AsUTF8AndSize(method, NULL);
 			if (uri_c == NULL || method_c == NULL)
 				goto error;
@@ -982,19 +1050,29 @@ static PyObject *__pyfly_configure(__pyfly_server_t *self, PyObject *args)
 			if (!PyCallable_Check(func))
 				goto error;
 
-			if (fly_register_route(ctx->route_reg, pyfly_route_handler, uri_c, *m, FLY_ROUTE_FLAG_PYTHON, (void *) func) == -1){
-				PyErr_SetString(PyExc_ValueError, "fly route register error");
+			struct fly_err err;
+			if (fly_register_route(ctx->route_reg, pyfly_route_handler, (char *) uri_c, uri_size, *m, FLY_ROUTE_FLAG_PYTHON, (void *) func, &err) != FLY_REGISTER_ROUTE_SUCCESS){
+				PyErr_Format(PyExc_ValueError, "Register route error. %s", err.content);
 				goto error;
 			}
 		}
 	}
 
-	self->master = master;
-	self->port = (long) fly_server_port();
-	self->host = (const char *) fly_server_host();
-	self->worker = (long) master->now_workers;
-	self->reqworker = (long) master->req_workers;
-	self->ssl = (bool) fly_ssl();
+	self->master			= master;
+	self->port				= (long) fly_server_port();
+	self->host 				= (const char *) fly_server_host();
+	self->worker			= (long) master->now_workers;
+	self->reqworker			= (long) master->req_workers;
+	self->ssl				= (bool) fly_ssl();
+	self->log_stdout		= fly_log_stdout();
+	self->log_stderr		= fly_log_stderr();
+	self->backlog			= fly_backlog();
+	self->max_response_content_length = fly_response_content_max_length();
+	self->max_request_content_length = fly_max_request_content_length();
+	self->request_timeout	= fly_request_timeout();
+	self->index_path		= fly_index_path();
+	self->default_content_path = fly_default_content_path();
+	self->encoding_threshold = fly_encode_threshold();
 
 #ifdef DEBUG
 	printf("%d\n", self->ssl);
@@ -1006,29 +1084,48 @@ static PyObject *__pyfly_configure(__pyfly_server_t *self, PyObject *args)
 		printf("%s\n", fly_log_path());
 #endif
 	if (self->ssl && fly_ssl_crt_path() != NULL){
-		self->ssl_crt_path = realpath((const char *) fly_ssl_crt_path(), NULL);
-		if (self->ssl_crt_path == NULL){
+		char *__tmp;
+
+		__tmp = realpath((const char *) fly_ssl_crt_path(), NULL);
+		if (__tmp == NULL){
 			PyErr_Format(PyExc_ValueError, "SSL certificate path: %s", strerror(errno));
 			goto error;
 		}
+		self->ssl_crt_path = fly_pballoc(self->master->context->pool, sizeof(char)*(strlen(__tmp)+1));
+		memset(self->ssl_crt_path, '\0', sizeof(char)*(strlen(__tmp)+1));
+		memcpy(self->ssl_crt_path, __tmp, sizeof(char)*strlen(__tmp));
+		free(__tmp);
 	}else
 		self->ssl_crt_path = NULL;
 
 	if (self->ssl && fly_ssl_key_path() != NULL){
-		self->ssl_key_path = realpath((const char *) fly_ssl_key_path(), NULL);
-		if (self->ssl_key_path == NULL){
+		char *__tmp;
+
+		__tmp = realpath((const char *) fly_ssl_key_path(), NULL);
+		if (__tmp == NULL){
 			PyErr_Format(PyExc_ValueError, "SSL key path:%s", strerror(errno));
 			goto error;
 		}
+		self->ssl_key_path = fly_pballoc(self->master->context->pool, sizeof(char)*(strlen(__tmp)+1));
+		memset(self->ssl_key_path, '\0', sizeof(char)*(strlen(__tmp)+1));
+		memcpy(self->ssl_key_path, __tmp, sizeof(char)*strlen(__tmp));
+		free(__tmp);
 	}else
 		self->ssl_key_path = NULL;
 
 	if (fly_log_path() != NULL){
-		self->log = realpath((const char *) fly_log_path(), NULL);
-		if (self->log == NULL){
+		char *__tmp;
+
+		__tmp = realpath((const char *) fly_log_path(), NULL);
+		if (__tmp == NULL){
 			PyErr_Format(PyExc_ValueError, "Log path: %s", strerror(errno));
 			goto error;
 		}
+
+		self->log = fly_pballoc(self->master->context->pool, sizeof(char)*(strlen(__tmp)+1));
+		memset(self->log, '\0', sizeof(char)*(strlen(__tmp)+1));
+		memcpy(self->log, __tmp, sizeof(char)*strlen(__tmp));
+		free(__tmp);
 	}else
 		self->log = NULL;
 
