@@ -1,12 +1,16 @@
 #ifndef _SERVER_H
 #define _SERVER_H
 
+#ifndef _GNU_SOURCE
+#define _GNU_SOURCE
+#endif
 #include <unistd.h>
 #include <stdlib.h>
 #include <sys/ioctl.h>
 #include <sys/socket.h>
 #include <netdb.h>
 #include "version.h"
+#include "err.h"
 #include "method.h"
 
 #define FLY_IP_V4			4
@@ -29,7 +33,8 @@ struct fly_sockinfo{
 typedef struct fly_sockinfo fly_sockinfo_t;
 #define FLY_SOCKET_OPTION		1
 struct fly_context;
-int fly_socket_init(struct fly_context *ctx, int port, fly_sockinfo_t *info, int flag);
+struct fly_err;
+int fly_socket_init(fly_context_t *ctx, int port, fly_sockinfo_t *info, int flag, struct fly_err *err);
 int fly_socket_release(int sockfd);
 
 #define FLY_PORT				"FLY_PORT"
@@ -45,6 +50,10 @@ int fly_socket_nonblocking(fly_sock_t s);
 int fly_socket_close(int fd, int how);
 int fly_server_port(void);
 char *fly_server_host(void);
-int fly_backlog(void);
+long fly_backlog(void);
+
+#ifdef DEBUG
+int fly_can_recv(int fd);
+#endif
 
 #endif
